
#include "GlutWindow.h"
#include "ArcBall.h"

#include <math.h>
#include <GL/glut.h>

#define DEG2RAD(a) ((a)/57.295)

#define ZOOM_MIN (0.05)
#define ZOOM_MAX (10.0)

#define STEREO_ANAGLYPH
#define STEREO_ANGLE (2.0)

CGlutWindow::CGlutWindow(void)
{
	m_pArcBall = new CArcBall();

	glutInitWindowSize (800,600); 
	glutInitWindowPosition (100, 100);
	glutCreateWindow ("GLUT Window");

	m_dFieldOfView = 30.0;
	m_nWidth = 1;
	m_nHeight = 1;

	m_dRadius = 1.5;
	m_vecCameraPosition.set(0.0, 0.0, 0.0,1.0);

	m_dCenter = m_dRadius/ sin(DEG2RAD(m_dFieldOfView)/2.0);

	m_dZoom = 1.0;
	m_dTranslateX = 0.0;
	m_dTranslateY = 0.0;
	m_nMouseState = 0;
	m_bStereoAnaglyph = false;
	m_dStereoAngle = STEREO_ANGLE;

	initializeGL();
}

CGlutWindow::~CGlutWindow(void)
{
	if (NULL != m_pArcBall) delete m_pArcBall;
}


void CGlutWindow::renderFrame() {

	double dAspectRatio = double(m_nWidth)/double(m_nHeight);

	// The usual OpenGL stuff to clear the screen and set up viewing.
	glClearColor(.5, .5, 1., 1.);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	GLfloat fTop, fRight, fNear, fFar;

	fNear   = float(m_dCenter - m_dRadius);
	fFar    = float(m_dCenter + m_dRadius);

	if (dAspectRatio > 1.0) {
		fRight = fNear * float(tan(DEG2RAD(m_dFieldOfView)/2.0) /m_dZoom);
		fTop   = fRight * float(dAspectRatio);		
	} else {
		fTop   = fNear * float(tan(DEG2RAD(m_dFieldOfView)/2.0) /m_dZoom);
		fRight = fTop / float(dAspectRatio);
	} 

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum( -fTop, fTop, -fRight, fRight, fNear, 10*fFar);

	CMatrix matModel = m_pArcBall->getRotatonMatrix().getInverse();

	double matrix[16];
	matModel.get(matrix[ 0], matrix[ 4], matrix[ 8], matrix[12],
		matrix[ 1], matrix[ 5], matrix[ 9], matrix[13],
		matrix[ 2], matrix[ 6], matrix[10], matrix[14],
		matrix[ 3], matrix[ 7], matrix[11], matrix[15]);


	glClearColor(0.0,0.0,0.0,1.0);
	glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
	glClear(GL_COLOR_BUFFER_BIT);

	if (m_bStereoAnaglyph) {
		glMatrixMode(GL_MODELVIEW);

		glLoadIdentity();
		glTranslated(m_dTranslateX * m_dRadius/m_dZoom, m_dTranslateY*m_dRadius/m_dZoom, -m_dCenter);
		glRotated(m_dStereoAngle,0.0,1.0,0.0);
		glMultMatrixd(matrix);
		glTranslated(m_vecCameraPosition[0], m_vecCameraPosition[1], m_vecCameraPosition[2]);

		glColorMask(GL_TRUE,GL_FALSE,GL_FALSE,GL_FALSE);
		glClear(GL_DEPTH_BUFFER_BIT);
		renderGeometry();

		glLoadIdentity();
		glTranslated(m_dTranslateX * m_dRadius/m_dZoom, m_dTranslateY*m_dRadius/m_dZoom, -m_dCenter);
		glRotated(-m_dStereoAngle,0.0,1.0,0.0);
		glMultMatrixd(matrix);
		glTranslated(m_vecCameraPosition[0], m_vecCameraPosition[1], m_vecCameraPosition[2]);
		glColorMask(GL_FALSE,GL_TRUE,GL_TRUE,GL_FALSE);
		glClear(GL_DEPTH_BUFFER_BIT);
		renderGeometry();

	} else {

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glTranslated(m_dTranslateX * m_dRadius/m_dZoom, m_dTranslateY*m_dRadius/m_dZoom, -m_dCenter);
		glMultMatrixd(matrix);
		glTranslated(m_vecCameraPosition[0], m_vecCameraPosition[1], m_vecCameraPosition[2]);

		renderGeometry();
	}

	glutSwapBuffers();
}


void CGlutWindow::resize(int width, int height) {
	m_nWidth = width; 
	m_nHeight = height;
	glViewport(0, 0, (GLsizei) width, (GLsizei) height);

	m_pArcBall->place(CPosition(width/2, height/2), 
		0.5 * sqrt(double(width * width + height * height)));
}

void CGlutWindow::keyEvent(unsigned char key,int x,int y){
	switch (key) {
		case '1':
			{
				glPointSize(1.0);
			}
			break;
		case '2':
			{
				glPointSize(2.0);
			}
			break;
		case '3':
			{
				glPointSize(3.0);
			}
			break;
		case '4':
			{
				glPointSize(4.0);
			}
			break;
		case 's':
			{
				m_bStereoAnaglyph = !m_bStereoAnaglyph;
			}
			break;
		case '+':
			{
				m_dStereoAngle += 0.25;
			}
			break;
		case '-':
			{
				m_dStereoAngle -= 0.25;
				if (m_dStereoAngle < 0.25) m_dStereoAngle = 0.25;
			}
			break;
		default:
			break;

	}

	glutPostRedisplay();

}

void CGlutWindow::mouseButtonEvent(int button, int state,int x,int y){

	switch(state) {
		case GLUT_DOWN:
			{
				m_vecMouseDown.set(double(x)/double(m_nWidth),double(y)/double(m_nHeight));

				switch (button) {
		case GLUT_LEFT_BUTTON:
			{
				m_nMouseState |= (MBUTTON_LEFT);
				m_pArcBall->setMouse( CPosition(x, m_nHeight-1-y));
				m_pArcBall->startDragging();
			}
			break;
		case GLUT_RIGHT_BUTTON:
			{
				m_nMouseState |= (MBUTTON_RIGHT);	
				m_bDragging = false;
				m_bZooming = true;
			}
			break;
		case GLUT_MIDDLE_BUTTON:
			{
				m_nMouseState |= (MBUTTON_MIDDLE);						
				m_bDragging = true;
				m_bZooming = false;
			}
			break;
		default:
			break;
				}
				break;
		case GLUT_UP:
			{
				switch (button) {
		case GLUT_LEFT_BUTTON:
			m_nMouseState ^= MBUTTON_LEFT;
			m_pArcBall->finishDragging();
			break;
		case GLUT_RIGHT_BUTTON:
			m_nMouseState ^= MBUTTON_RIGHT;
			break;
		case GLUT_MIDDLE_BUTTON:
			m_nMouseState ^= MBUTTON_MIDDLE;
			break;
		default:
			break;
				}
				m_bDragging = false;
				m_bZooming  = false;
			}
			}


	}
}

void CGlutWindow::mouseMoveEvent(int x,int y){


	CPosition pos(double(x)/double(m_nWidth),double(y)/double(m_nHeight));

	CPosition drag = pos-m_vecMouseDown;

	if (m_nMouseState & (MBUTTON_LEFT)) {
		m_pArcBall->setMouse(CPosition(x, m_nHeight-1- y));
		glutPostRedisplay();

	} else if ((m_bDragging) && (m_nMouseState & MBUTTON_MIDDLE)) {

		m_dTranslateX +=  2.0 * drag[0];
		m_dTranslateY += -2.0 * drag[1];

		if (m_dTranslateX >  m_dZoom) m_dTranslateX =  m_dZoom;
		if (m_dTranslateX < -m_dZoom) m_dTranslateX = -m_dZoom;
		if (m_dTranslateY >  m_dZoom) m_dTranslateY =  m_dZoom;
		if (m_dTranslateY < -m_dZoom) m_dTranslateY = -m_dZoom;
		glutPostRedisplay();
	} else if ((m_bZooming) && (m_nMouseState & MBUTTON_RIGHT )) {

		double dZoom = 10.0*drag[0]; 
		m_dZoom += dZoom;
		if (m_dZoom < ZOOM_MIN) m_dZoom = ZOOM_MIN;
		if (m_dZoom > ZOOM_MAX) m_dZoom = ZOOM_MAX;
		if (m_dTranslateX >  m_dZoom) m_dTranslateX =  m_dZoom;
		if (m_dTranslateX < -m_dZoom) m_dTranslateX = -m_dZoom;
		if (m_dTranslateY >  m_dZoom) m_dTranslateY =  m_dZoom;
		if (m_dTranslateY < -m_dZoom) m_dTranslateY = -m_dZoom;
		glutPostRedisplay();
	} 
	m_vecMouseDown = pos;
}

void CGlutWindow::idle(){
}

void CGlutWindow::initializeGL()
{

#ifdef LIGHTING
	GLfloat ambient[] =
	{0.4f, 0.4f, 0.4f, 1.0f};
	GLfloat diffuse[] =
	{1.0f, 1.0f, 1.0f, 1.0f};
	GLfloat specular[] =
	{0.5f, 0.5f, 0.5f, 1.0f};
	GLfloat position[] =
	{1.0f, 1.0f, 1.0f, 1.0f};

	GLfloat lmodel_ambient[] =
	{0.2f, 0.2f, 0.2f, 1.0f};
	GLfloat local_view[] =
	{0.0f};

	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
	glLightfv(GL_LIGHT0, GL_POSITION, position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 1);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
#endif

	glEnable(GL_DEPTH_TEST);

	glClearColor(1.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_SMOOTH);

}


void CGlutWindow::renderGeometry() {

}