#include "ViewerWindow.h"
#include <time.h>
#include <GL/glut.h>

#include <GL/glext.h>

#include <assert.h>

#include <iostream>
#include <fstream>
using namespace std;

// openGL extensions for vertex buffers
PFNGLBINDBUFFERPROC    glBindBuffer = NULL;
PFNGLDELETEBUFFERSPROC glDeleteBuffers = NULL;
PFNGLGENBUFFERSPROC    glGenBuffers = NULL;
PFNGLBUFFERDATAPROC    glBufferData = NULL;

#define BUFFER_OFFSET(i) ((char *)NULL + (i))

#define GEOMETRY_FILENAME "julia_sascha.raw"
#define TEXTURE_FILENAME "julia_sascha_bmp.raw"

#define CLAMP(a,b,c) { b = ((b) < (a))? (a) : (((b) > (c))? (c): (b));}

long getFileSize(const char *filename) {

	long begin,end;
	ifstream myfile(filename);
	begin = myfile.tellg();
	myfile.seekg (0, ios::end);
	end = myfile.tellg();
	myfile.close();
	long nSize = (end-begin);
	return nSize;

}



CViewerWindow::CViewerWindow()
{
	m_nVertexBufferId = 0;
	m_nTextureId = 0;
	m_nNumVertices = 0;

	m_dZoom = 2.0;
}

CViewerWindow::~CViewerWindow(void)
{
	
}

void CViewerWindow::initScene() {

	//*************************************************************************************
	// init opengl extension function pointers
	if (NULL == glBindBuffer) {

#ifdef WIN32
		// for Microsoft Windows
		glBindBuffer    = (PFNGLBINDBUFFERPROC   ) wglGetProcAddress("glBindBuffer");
		glDeleteBuffers = (PFNGLDELETEBUFFERSPROC) wglGetProcAddress("glDeleteBuffers");
		glGenBuffers    = (PFNGLGENBUFFERSPROC   ) wglGetProcAddress("glGenBuffers");
		glBufferData    = (PFNGLBUFFERDATAPROC   ) wglGetProcAddress("glBufferData");
#else
		// for X Windows (linux/Unix/MacOS etc.)
		glBindBuffer = glxGetProcAddress("glBindBuffer");
		glDeleteBuffers = glxGetProcAddress("glDeleteBuffers");
		glGenBuffers = glxGetProcAddress("glGenBuffers");
		glBufferData = glxGetProcAddress("glBufferData");
#endif


	}

	//*************************************************************************************
	// init vertex buffer
	long nSizeGeo = getFileSize(GEOMETRY_FILENAME);
	assert(nSizeGeo != 0);

	FILE *pFileGeo = fopen(GEOMETRY_FILENAME,"rb");
	assert(pFileGeo != NULL);
	float *pVertexBuffer = (float *) new char[nSizeGeo];
	assert(pVertexBuffer != NULL);

	m_nNumVertices = nSizeGeo / (5 * sizeof(float));
	assert( nSizeGeo == fread(pVertexBuffer,sizeof(char),nSizeGeo,pFileGeo));
	fclose(pFileGeo);
	
	glGenBuffers(1, & m_nVertexBufferId);
	glBindBuffer(GL_ARRAY_BUFFER_ARB, m_nVertexBufferId);
	glBufferData(GL_ARRAY_BUFFER_ARB, m_nNumVertices * 5 * sizeof(float), pVertexBuffer, GL_STATIC_DRAW_ARB);
	
	delete [] pVertexBuffer;


	//*************************************************************************************
	// init texture
	long nSizeTex = getFileSize(TEXTURE_FILENAME);
	assert(nSizeTex != 0);

	FILE *pFileTex = fopen(TEXTURE_FILENAME,"rb");
	assert(pFileTex != NULL);
	char *pTextureBuffer = (char *) new char[nSizeTex];
	assert(pTextureBuffer != NULL);
	assert( nSizeTex == fread(pVertexBuffer,sizeof(char),nSizeTex,pFileTex));
	fclose(pFileTex);

	
	int nNumTexels = nSizeTex/3; // RGB
	// i assume a square texture
	int nTexRes = 1;
	while(nTexRes != nNumTexels) {
		nNumTexels = nNumTexels >> 1;
		nTexRes = nTexRes << 1;
	}
	
	glGenTextures(1, &m_nTextureId);
	glBindTexture(GL_TEXTURE_2D,m_nTextureId);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,nTexRes,nTexRes,0,GL_RGB,GL_UNSIGNED_BYTE,pTextureBuffer);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	delete [] pTextureBuffer;
}

void
CViewerWindow::renderGeometry() {

	static bool bOnce = false;
	if (bOnce  != true) {
		initScene();
		bOnce = true;
	}

	glDisable(GL_TEXTURE_2D);
	glEnableClientState(GL_VERTEX_ARRAY); 
	glVertexPointer(3, GL_FLOAT, 5*sizeof(float), BUFFER_OFFSET(0));

	glEnable(GL_TEXTURE_2D);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY); 
	glTexCoordPointer(2, GL_FLOAT, 5*sizeof(float), BUFFER_OFFSET(3*sizeof(float)));

	glDrawArrays(GL_POINTS,0, m_nNumVertices);

	glBindTexture(GL_TEXTURE_2D,m_nTextureId);
	glColor3f(1.0,1.0,1.0);

	/*
	glBegin(GL_QUADS);
	glTexCoord2d(0.0,1.0);glVertex3f(-1.0, 1.0,0.0);
	glTexCoord2d(1.0,1.0);glVertex3f( 1.0, 1.0,0.0);
	glTexCoord2d(1.0,0.0);glVertex3f( 1.0,-1.0,0.0);
	glTexCoord2d(1.0,0.0);glVertex3f(-1.0,-1.0,0.0);
	glEnd();
	*/
	
}
